# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4

import http.client
import urllib.request, urllib.error, urllib.parse
import hashlib

from .utils import valid

from . import settings

import logging
logger = logging.getLogger('link')


class InvalidCertificateException(http.client.HTTPException, urllib.error.URLError):
    def __init__(self, fingerprint, cert, reason):
        http.client.HTTPException.__init__(self)
        self._fingerprint = fingerprint
        self._cert_fingerprint = hashlib.sha1(cert).hexdigest()
        self.reason = reason

    def __str__(self):
        return ('%s (local) != %s (remote) (%s)\n' %
                (self._fingerprint, self._cert_fingerprint, self.reason))

class FingerprintHTTPSConnection(http.client.HTTPSConnection):

    def __init__(self, host, port=None, fingerprint=None, check_hostname=None, **kwargs):
        self._fingerprint = fingerprint
        if self._fingerprint:
            check_hostname = None
        http.client.HTTPSConnection.__init__(self, host, port,
                check_hostname=check_hostname, **kwargs)

    def _check_fingerprint(self, cert):
        if len(self._fingerprint) == 40:
            fingerprint = hashlib.sha1(cert).hexdigest()
        elif len(self._fingerprint) == 64:
            fingerprint = hashlib.sha256(cert).hexdigest()
        elif len(self._fingerprint) == 128:
            fingerprint = hashlib.sha512(cert).hexdigest()
        else:
            logging.error('unkown _fingerprint length %s (%s)',
                self._fingerprint, len(self._fingerprint))
            return False
        return fingerprint == self._fingerprint

    def connect(self):
        http.client.HTTPSConnection.connect(self)
        if self._fingerprint:
            cert = self.sock.getpeercert(binary_form=True)
            if not self._check_fingerprint(cert):
                raise InvalidCertificateException(self._fingerprint, cert,
                                                  'fingerprint mismatch')
        #logger.debug('CIPHER %s VERSION %s', self.sock.cipher(), self.sock.ssl_version)

class FingerprintHTTPSHandler(urllib.request.HTTPSHandler):

    def __init__(self, debuglevel=0, context=None, check_hostname=None, fingerprint=None):
        urllib.request.AbstractHTTPHandler.__init__(self, debuglevel)
        self._context = context
        self._check_hostname = check_hostname
        self._fingerprint = fingerprint

    def https_open(self, req):
        return self.do_open(FingerprintHTTPSConnection, req,
            context=self._context, check_hostname=self._check_hostname,
            fingerprint=self._fingerprint)

def get_opener(fingerprint):
    handler = FingerprintHTTPSHandler(fingerprint=fingerprint)
    opener = urllib.request.build_opener(handler)
    return opener

class Response(object):
    headers = {}
    error = None
    body = None
    data = None
    user = None
    code = 200

def read(url, body=None, headers={}, fingerprint=None, timeout=None):
    if not timeout:
        timeout = settings.TIMEOUT
    if not body:
        body = None
    opener = get_opener(fingerprint)
    headers = dict(headers)
    request = urllib.request.Request(url, data=body, headers=headers)
    response = Response()
    logger.debug('open %s [%s]', url, fingerprint)
    logger.debug('headers: %s', headers)
    try:
        r = opener.open(request, timeout=timeout)
    except urllib.error.HTTPError as e:
        response.code = e.code
        if e.code >= 500:
            logger.debug('urllib.error.HTTPError %s %s', e, e.code)
            response.error = e
        else:
            response.headers = e.headers
            response.body = e.read()
        return response
    except urllib.error.URLError as e:
        logger.debug('urllib.error.URLError %s', e)
        response.error = e
        response.code = 500
        return response
    except:
        logger.debug('unknown url error', exc_info=1)
        response.error = 'unkown url error'
        response.code = 500
        return response

    response.headers = r.headers
    response.code = r.getcode()
    sig = r.headers.get('X-Ed25519-Signature')
    if sig:
        key = r.headers.get('X-Ed25519-Key')
        data = r.read()
        if valid(key, data, sig):
            response.data = data
            response.user = key
        else:
            response.error = 'Invalid Signature'
            response.code = 500
    else:
        response.body = r
    logger.debug('response headers: %s', dict(r.headers))
    return response

def node_url(node):
    host = node['host']
    port = node['port']
    if ':' in host:
        url = 'https://[%s]:%s' % (host, port)
    else:
        url = 'https://%s:%s' % (host, port)
    return url

def can_connect(data):
    r = read(node_url(data), fingerprint=data['cert'], timeout=1)
    if r.error:
        return False
    return True
